// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020-2024 Nicolas Pfeiffer, Florian Schmaus
#pragma once

#include <cstdint>
#include <ostream>

#define membar(call)                                            \
	do {                                                          \
		call;                                                       \
		asm("nop" ::: "rbx", "r12", "r13", "r14", "r15", "memory"); \
	} while (0);

class Continuation {
 public:
	void *bp;
	void *sp;
	void *ip;

	inline __attribute__((always_inline)) Continuation() : ip(nullptr) {
		asm volatile(
				"mov %%rbp, %0;"
				"mov %%rsp, %1;"
				: "=rm"(bp), "=rm"(sp)
				:
				:);
	};

	inline __attribute__((always_inline, noreturn)) void execute(const void *_sp) {
		asm("mov %0, %%rsp;"
				"mov %1, %%rbp;"
				"jmp *%2;"
				:
				: "r"(_sp), "r"(bp), "r"(ip)
				: "memory");
		__builtin_unreachable();
	};

	inline __attribute__((always_inline, returns_twice)) uintptr_t setJmp() {
		auto set_rip = [](Continuation * c) __attribute__((noinline, hot, optimize(3))) {
			c->ip = __builtin_return_address(0);
			return 0;
		};

		uintptr_t res;
		res = set_rip(this);
		// Same clobber list as member but with rax added, which will be
		// used to hold 'res' once we longjmp to this.
		asm("nop" ::: "rax", "rbx", "r12", "r13", "r14", "r15", "memory");

		return res;
	};

	inline __attribute__((always_inline, noreturn)) void longJmp(uintptr_t ret) {
		asm("mov %0, %%rsp;"
				"mov %1, %%rbp;"
				"jmp *%2;"
				:
				: "r"(sp), "r"(bp), "r"(ip), "a"(ret)
				: "memory");
		__builtin_unreachable();
	};

	void printTo(std::ostream &strm) const;

	friend auto operator<<(std::ostream &strm, const Continuation &continuation) -> std::ostream &;
};
