// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2021 Florian Schmaus, Florian Fischer
#pragma once

#include <cstdint>
#include <iostream>

namespace emper {
/**
 * @brief Descriptor for all the possible locations a Fiber can be obtained from
 *
 * This enum is used to collect stats, create hints where fibers are and
 * make informed notification or scheduling decisions.
 */
enum class FiberSource : uintptr_t {
	local,				 /*!< A worker's own work-stealing queue */
	mpscQueue,		 /*!< A worker's own mpsc queue (inbox / priority) */
	stolen,				 /*!< A other worker's work-stealing queue */
	io,						 /*!< A worker's own io_uring completion queue */
	ioStolen,			 /*!< A other worker's io_uring completion queue */
	anywhereQueue, /*!< The anywhere queue */
	hintWsq,			 /*!< A known other worker's work-stealing queue */
	hintIo,				 /*!< A known worker's io_uring completion queue */
	hintAq,				 /*!< Straight from the anywhere queue */
};

}	 // namespace emper

auto operator<<(std::ostream& os, const emper::FiberSource& fiberSource) -> std::ostream&;
