// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020-2021 Florian Schmaus
#include "Scheduler.hpp"

#include <stdexcept>

#include "CallerEnvironment.hpp"
#include "Common.hpp"
#include "Runtime.hpp"
#include "RuntimeStrategy.hpp"
#include "WakeupStrategy.hpp"

Scheduler::Scheduler(Runtime& runtime, RuntimeStrategy& strategy)
		: runtime(runtime), dispatcher(strategy.getDispatcher()) {}

void Scheduler::addNewWorkerHook(const std::function<void(workerid_t)>& hook) {
	runtime.addNewWorkerHook(hook);
}

template <CallerEnvironment callerEnvironment>
void Scheduler::wakeupSleepingWorkers(emper::FiberHint hint) {
	runtime.wakeupSleepingWorkers<callerEnvironment>(hint);
}

// show the compiler our template incarnations
template void Scheduler::wakeupSleepingWorkers<CallerEnvironment::EMPER>(emper::FiberHint hint);
template void Scheduler::wakeupSleepingWorkers<CallerEnvironment::ANYWHERE>(emper::FiberHint hint);

void Scheduler::scheduleOn(Fiber& fiber, workerid_t workerId) {
	// Calling scheduleOn() only works from within the EMPER runtime.
	emper::assertInRuntime();

	if (unlikely(workerId >= runtime.getWorkerCount()))
		throw std::runtime_error("WorkerId to big for worker count");

	LOGD("Scheduling fiber " << &fiber << " on worker " << workerId);
	scheduleOnInternal(fiber, workerId);
}

void Scheduler::notifyRuntimeAboutWorkStolen() { runtime.wakeupStrategy.onWorkStolen(); }
