// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020-2021 Florian Schmaus
#pragma once

#include <random>

#include "Common.hpp"
#include "FiberHint.hpp"
#include "emper-common.h"

class Runtime;

namespace emper::sleep_strategy {
class AbstractIoSleepStrategy;
}

class Worker {
 private:
	static thread_local Worker* currentWorker;

	ALIGN_TO_CACHE_LINE unsigned int seed;

	std::mt19937 mtGenerator;

	std::uniform_int_distribution<workerid_t> randomWorkerIdGenerator;

	const workerid_t workerId;

	emper::FiberHint dispatchHint;

	Worker(workerid_t workerId, workerid_t workerCount, unsigned int seed)
			: seed(seed),
				mtGenerator(seed),
				randomWorkerIdGenerator(0, workerCount - 1),
				workerId(workerId) {}

	void setWorker();

	friend Runtime;
	friend class AbstractWorkStealingScheduler;
	friend class emper::sleep_strategy::AbstractIoSleepStrategy;

 public:
	[[nodiscard]] auto getWorkerId() const -> workerid_t { return workerId; }

	// https://stackoverflow.com/a/3747462/194894
	inline auto nextRand() -> int {
		seed = 214013 * seed + 2531011;
		auto shifted_seed = seed >> 16;
		return *reinterpret_cast<int*>(&shifted_seed) & 0x7FFF;
	}

	inline auto nextRandomWorkerId() -> workerid_t { return randomWorkerIdGenerator(mtGenerator); }

	static inline auto isWorkerThread() -> bool { return currentWorker != nullptr; }

	static inline auto getCurrentWorker() -> Worker* { return currentWorker; }

	// https://stackoverflow.com/a/3747462/194894
	static inline auto rand() -> int {
		Worker* currentWorker = getCurrentWorker();
		return currentWorker->nextRand();
	}

	static inline auto getCurrentWorkerId() -> workerid_t {
		Worker* currentWorker = getCurrentWorker();
		return currentWorker->getWorkerId();
	}
};
