// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020-2022 Florian Schmaus
#include "DebugUtil.hpp"

#include <execinfo.h>
#include <unistd.h>

#include <csignal>
#include <cstdio>
#include <cstdlib>

#include "Common.hpp"

static void handler(int sig) {
	const int backtrace_size = 20;
	// NOLINTNEXTLINE(modernize-avoid-c-arrays)
	void* array[backtrace_size];

	int size = backtrace(array, backtrace_size);

	int res = fprintf(stderr, "Error: signal %d:\n", sig);
	if (res) DIE_MSG_ERRNO("fprintf() failed");

	backtrace_symbols_fd(array, size, STDERR_FILENO);
	exit(EXIT_FAILURE);
}

void enableStacktraceOnAborts() {
	sighandler_t prev_handler;
	prev_handler = signal(SIGSEGV, handler);
	if (prev_handler == SIG_ERR) DIE_MSG_ERRNO("signal() failed");
	prev_handler = signal(SIGABRT, handler);
	if (prev_handler == SIG_ERR) DIE_MSG_ERRNO("signal() failed");
}
