// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2021-2022 Florian Fischer
#pragma once

#include <cinttypes>
#include <cstdlib>
#include <limits>
#include <optional>
#include <ostream>
#include <string>
#include <typeinfo>

#include "Common.hpp"

namespace emper::lib::env {

auto getBoolFromEnv(const std::string& key) -> std::optional<bool>;

template <typename unsigned_type>
static auto getUnsignedFromEnv(const std::string& key) -> std::optional<unsigned_type> {
	char* envVar = std::getenv(key.c_str());
	if (!envVar) {
		return std::nullopt;
	}

	std::string envStr(envVar);
	char* last;

	uintmax_t num = std::strtoumax(envStr.c_str(), &last, 10);
	if (last != &envStr[0] + envStr.size()) {
		DIE_MSG(key << " has invalid value: " << envStr << " (expected base-10 number)");
	}

	const unsigned_type t_max = std::numeric_limits<unsigned_type>::max();
	if (num > static_cast<uintmax_t>(t_max)) {
		DIE_MSG(key << " is to big: " << envStr << " (type '" << typeid(t_max).name()
								<< "' max: " << t_max << ")");
	}
	return static_cast<unsigned_type>(num);
}

}	 // namespace emper::lib::env
