// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2021-2022 Florian Fischer
#include "sleep_strategy/PipeSleepStrategy.hpp"

#include <atomic>
#include <cstdint>

#include "io/IoContext.hpp"

using emper::io::IoContext;

namespace emper::sleep_strategy {
void PipeSleepStrategy::sleep() {
	LOGD("going to sleep");
	IoContext& io = *IoContext::getWorkerIo();

	auto customSleepLogic = [this, &io] {
		// increment the sleeper count if it was negative we should skip sleeping
		int64_t sleeping = this->sleepers.fetch_add(1, std::memory_order_acquire);
		if (sleeping < 0) {
			LOGD("observed sleeper count as: " << sleeping << " -> skip sleeping");
			return false;
		}
		prepareSleepRead(io, this->global.sleepFd);
		LOGD("prepared sleepFd read and set sleepers count to: " << sleeping + 1);
		return true;
	};

	const bool shouldSleep = prepareSleep(customSleepLogic);
	if (!shouldSleep) return;
	submitAndWaitForCompletionsIn(io);
}
}	 // namespace emper::sleep_strategy
