// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2021 Florian Schmaus
#pragma once

#include <atomic>
#include <cstdint>
#include <iostream>
#include <vector>

#include "emper-common.h"

enum class LogSubsystem;
template <LogSubsystem logSubsystem>
class Blockable;	// IWYU pragma: keep
class Runtime;
namespace emper {
enum class BlockablePurpose;
}	 // namespace emper

namespace emper::stats {

class FromAnywhere {
 private:
	std::vector<std::atomic<uint32_t>> unblockAffinitiesGeneric;
	std::vector<std::atomic<uint32_t>> unblockAffinitiesIo;

	FromAnywhere(workerid_t workerCount);

	void recordUnblockedContext(workeraffinity_t workerAffinity, BlockablePurpose blockablePurpose);

	auto operator+=(const FromAnywhere& other) -> FromAnywhere&;

 public:
	void print(std::ostream& out = std::cout);

	template <LogSubsystem>
	friend class ::Blockable;
	friend class ::Runtime;
};

}	 // namespace emper::stats
