// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2021 Florian Schmaus
#include "strategies/AbstractWorkStealingStats.hpp"

#include <iostream>
#include <string>

#include "WorkerLocalData.hpp"
#include "lib/math.hpp"
#include "strategies/AbstractWorkStealingStrategy.hpp"

AbstractWorkStealingStats::AbstractWorkStealingStats(AbstractWorkStealingStrategy& strategy)
		: workerStats(strategy.allWorkerStats.getSnapshot()) {
	for (auto& workerStat : workerStats) {
		comulatedWorkerStats += workerStat;
	}
}

void AbstractWorkStealingStats::print(std::ostream& out) {
	out << "total-scheduled-fibers-to-local: "
			<< std::to_string(comulatedWorkerStats.scheduledFibersToLocal) << std::endl
			<< "total-scheduled-fibers-to-overflow-queue: "
			<< std::to_string(comulatedWorkerStats.scheduledFibersToOverflowQueue) << std::endl
			<< "total-scheduled-fibers-to-mpsc-queue: "
			<< std::to_string(comulatedWorkerStats.scheduledFibersToMpscQueue) << std::endl
			<< "global-max-queue-length: " << std::to_string(comulatedWorkerStats.maxQueueLength)
			<< std::endl
			<< "total-next-fiber-from-local: " << std::to_string(comulatedWorkerStats.nextFiberFromLocal)
			<< std::endl
			<< "total-next-fiber-hint-local: "
			<< std::to_string(comulatedWorkerStats.nextFiberFromHintLocal) << std::endl
			<< "total-next-fiber-hint-anywherequeue: "
			<< std::to_string(comulatedWorkerStats.nextFiberFromHintAnywhere) << std::endl
			<< "total-next-fiber-hint-io: " << std::to_string(comulatedWorkerStats.nextFiberFromHintIo)
			<< std::endl
			<< "total-next-fiber-stolen: " << std::to_string(comulatedWorkerStats.nextFiberStolen)
			<< std::endl
			<< "total-next-io-fiber-stolen: " << std::to_string(comulatedWorkerStats.nextIoFiberStolen)
			<< std::endl
			<< "total-next-fiber-from-anywhere-queue: "
			<< std::to_string(comulatedWorkerStats.nextFiberFromAnywhereQueue) << std::endl
			<< "total-fibers-lifted-from-anywhere-queue: "
			<< std::to_string(comulatedWorkerStats.totalFibersLiftedFromAnywhereQueue) << std::endl
			<< "max-fibers-lifted-from-anywhere-queue: "
			<< std::to_string(comulatedWorkerStats.maxFibersLiftedFromAnywhereQueue) << std::endl
			<< "avg-fibers-lifted-from-anywhere-queue: "
			<< std::to_string(comulatedWorkerStats.avgFibersLiftedFromAnywhereQueue.getAverage())
			<< std::endl;
}
