// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020-2021 Florian Schmaus
#pragma once

#include <cstddef>
#include <cstdint>

#include "lib/math.hpp"

namespace math = emper::lib::math;

class AbstractWorkStealingWorkerStats {
	friend class AbstractWorkStealingScheduler;

 public:
	uint64_t scheduledFibersToLocal = 0;
	uint64_t scheduledFibersToOverflowQueue = 0;
	uint64_t scheduledFibersToMpscQueue = 0;
	uint64_t maxQueueLength = 0;
	uint64_t nextFiberFromLocal = 0;
	uint64_t nextFiberFromHintLocal = 0;
	uint64_t nextFiberFromHintAnywhere = 0;
	uint64_t nextFiberFromHintIo = 0;
	uint64_t nextFiberStolen = 0;
	uint64_t nextIoFiberStolen = 0;
	uint64_t nextFiberFromAnywhereQueue = 0;

	uint64_t totalFibersLiftedFromAnywhereQueue = 0;
	uint64_t maxFibersLiftedFromAnywhereQueue = 0;
	math::RunningAverage<double, uint32_t> avgFibersLiftedFromAnywhereQueue;

	auto operator+=(const AbstractWorkStealingWorkerStats& other) -> AbstractWorkStealingWorkerStats&;

 private:
	void recordFibersLiftedFromAnywhereQueue(size_t fibersLiftedFromAnywhere);

	void recordScheduledToLocalAndQueueLength(uint64_t queueLength);
};
