// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020-2021 Florian Schmaus
#pragma once

#include <optional>

#include "CallerEnvironment.hpp"
#include "FiberHint.hpp"
#include "FiberSource.hpp"
#include "NextFiberResult.hpp"
#include "emper-common.h"
#include "strategies/AbstractWorkStealingScheduler.hpp"

class Fiber;
class Runtime;
class RuntimeStrategy;

class WsScheduler : public AbstractWorkStealingScheduler {
 protected:
	void scheduleInternal(Fiber& fiber) override { scheduleViaWorkStealing(fiber); }
	void scheduleOnInternal(Fiber& fiber, workerid_t workerId) override {
		scheduleToMpscQueue(fiber, workerId);
	}
	void scheduleFromAnywhereInternal(Fiber& fiber) override {
		enqueueInAnywhereQueue(fiber);
		onNewWork<CallerEnvironment::ANYWHERE>(emper::FiberHint{emper::FiberSource::anywhereQueue});
	}

	void scheduleFromAnywhereInternal(Fiber** fibers, unsigned count) override {
		insertInAnywhereQueue(fibers, count);
		onNewWork<CallerEnvironment::ANYWHERE>(emper::FiberHint{emper::FiberSource::anywhereQueue});
	}

 public:
	WsScheduler(Runtime& runtime, RuntimeStrategy& strategy);

	auto nextFiber() -> std::optional<NextFiberResult> override {
		return nextFiberResultFromMpscQueueOrWorkStealing();
	};
};
