// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020-2022 Florian Schmaus
#include "WsStrategy.hpp"

#include "WsDispatcher.hpp"
#include "WsScheduler.hpp"
#include "strategies/ws/WsStrategyStats.hpp"	// for WsStrategyStats

class Runtime;
class RuntimeStrategyStats;

auto WsStrategy::getFactory() -> std::function<WsStrategy*(Runtime&)> {
	static std::function<WsStrategy*(Runtime&)> factory = [](Runtime& runtime) {
		return new WsStrategy(runtime);
	};
	return factory;
}

WsStrategy::WsStrategy(Runtime& runtime)
		: AbstractWorkStealingStrategy(runtime), scheduler(runtime, *this), dispatcher(runtime) {}

auto WsStrategy::getScheduler() -> WsScheduler& { return scheduler; }

auto WsStrategy::getDispatcher() -> WsDispatcher& { return dispatcher; }

auto WsStrategy::getStats() -> std::shared_ptr<RuntimeStrategyStats> {
	return std::make_shared<WsStrategyStats>(*this);
}
