// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020-2021 Florian Fischer
#include <cstddef>
#include <functional>
#include <string>	 // for string, allocator
#include <vector>	 // for vector

#include "Common.hpp"
#include "CountingPrivateSemaphore.hpp"	 // for CPS
#include "emper.hpp"										 // for async, spawn
#include "fixtures/network.hpp"					 // for echo_client, echo_serve
#include "io.hpp"												 // for tcp_listener

class Fiber;

void emperTest() {
	std::string port = "4242";
	std::string addr = "127.0.0.1";
	const size_t MAX = 1024;

	CPS cps;
	Fiber* listener =
			emper::io::tcp_listener(addr, port, echo_serve, 1024, {emper::io::SockOpt::ReusePort});
	if (!listener) {
		DIE_MSG("failed to create listener");
	}
	async(listener);

	spawn(
			[&] {
				const std::vector<std::string> strings = {"foo", "bar", std::string(MAX, 'a'), "quit\n"};
				echo_client(addr, port, strings);
			},
			cps);

	cps.wait();
}
